<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/

/**
* Class geodb_object
*
* Represents georeferenced data. Latitude, longitude and name
* of the location are the basic information.
*
* @access   public
* @package  Geo
*/

class geodb_object {

	/**
	* Name
	*
	* @var  string
	*/
	var $name;

	/**
	* Latitude (degrees)
	*
	* @var  float
	*/
	var $latitude;

	/**
	* Latitude (degrees)
	*
	* @var  float
	*/
	var $longitude;

	/**
	* Latitude RAD
	*
	* @var  float
	*/
	var $latitudeRad;

	/**
	* Longitude RAD
	*
	* @var  float
	*/
	var $longitudeRad;

	/**
	* Latitude DMS
	*
	* @var  string
	*/
	var $latitudeDMS;

	/**
	* Longitude DMS (degrees/minutes/seconds)
	*
	* @var  string
	*/
	var $longitudeDMS;

	/**
	* Database values
	*
	* @var         array
	*/
	var $dbValues = array ();

	/**
	* Constructor geodb_object
	*
	* $latitude and $longitude absolute > PI, otherwise there is an auto-detection
	*
	* @access  private
	* @param   string    $name       name of the location
	* @param   float     $latitude   latitude given as radiant or degree
	* @param   float     $longitude  longitude given as radiant or degree
	* @param   boolean   $degree     false by default, has to be set to true if
	* @param   array     $dbValues   database values, specific to the source implementation
	* @return  void
	*/

	function geodb_object ($name = '', $latitude = 0.0, $longitude = 0.0, $degree = false, $dbValues = array () ) {

		global $cfgStrings;

		$this->name = $name;
		$this->dbValues = $dbValues;
		if (strstr ($latitude, ' ') && strstr ($longitude, ' ') ) {
			$this->latitude = geodb::dms2deg ($latitude);
			$this->longitude = geodb::dms2deg ($longitude);
			$this->latitudeRad = deg2rad ($this->latitude);
			$this->longitudeRad = deg2rad ($this->longitude);
		} else {
			if ( (abs ($latitude)>M_PI) || (abs ($longitude)>M_PI) ) {
				$degree = true;
			}
			if ($degree) {
				$this->latitude = $latitude;
				$this->longitude = $longitude;
				$this->latitudeRad = deg2rad ($this->latitude);
				$this->longitudeRad = deg2rad ($this->longitude);
			} else {
				$this->latitude = rad2deg ($latitude);
				$this->longitude = rad2deg ($longitude);
				$this->latitudeRad = $latitude;
				$this->longitudeRad = $longitude;
			}
		}
		$this->latitudeDMS = ($this->latitude>0? $cfgStrings['ORIENTATION_SHORT'][0] : $cfgStrings['ORIENTATION_SHORT'][3]) . ' ' . geodb::deg2dms ($this->latitude);
		$this->longitudeDMS = ($this->longitude>0? $cfgStrings['ORIENTATION_SHORT'][2] : $cfgStrings['ORIENTATION_SHORT'][5]) . ' ' . geodb::deg2dms ($this->longitude);

	}

	/**
	* Distance between this and a given GeoObject (float)
	*
	* Returns the distance between an overgiven and this
	* GeoObject in the passed unit as float.
	*
	* @access  public
	* @param   object  &$geoObject  GeoObject
	* @param   int     $unit        please use _GEODB_UNIT_* constants
	* @return  float
	* @see     getDistanceString()
	*/

	function getDistance (&$geoObject, $unit = _GEODB_UNIT_DEFAULT) {

		$earthRadius = geodb::getEarthRadius ($unit);
		return acos ( (sin ($this->latitudeRad)*sin ($geoObject->latitudeRad) )+ (cos ($this->latitudeRad)*cos ($geoObject->latitudeRad)*cos ($this->longitudeRad- $geoObject->longitudeRad) ) )* $earthRadius;

	}

	/**
	* Distance between this and the passed GeoObject (string)
	*
	* Returns the distance between an overgiven and this GeoObject
	* rounded to 2 decimal places. The passed unit is returned at
	* the end of the sting.
	*
	* @access  public
	* @param   object   &$geoObject  GeoObject
	* @param   int      $unit        please use _GEODB_UNIT_* constants
	* @return  string
	* @see     getDistance()
	*/

	function getDistanceString (&$geoObject, $unit = _GEODB_UNIT_DEFAULT) {

		global $cfgStrings;
		return round ($this->getDistance ($geoObject, $unit), 2) . ' ' . $cfgStrings[$unit];

	}

	/**
	* north-south distance between this and the passed GeoObject
	*
	* Returns the north-south distance between this and the passed
	* object in the passed unit as float.
	*
	* @access  public
	* @param   object  &$geoObject  GeoObject
	* @param   int     $unit        please use _GEODB_UNIT_* constants
	* @return  float
	* @see     getDistanceWE()
	*/

	function getDistanceNS (&$geoObject, $unit = _GEODB_UNIT_DEFAULT) {

		$earthRadius = geodb::getEarthRadius ($unit);
		if ($this->latitudeRad>$geoObject->latitudeRad) {
			$direction = -1;
		} else {
			$direction = 1;
		}
		return $direction*acos ( (sin ($this->latitudeRad)*sin ($geoObject->latitudeRad) )+ (cos ($this->latitudeRad)*cos ($geoObject->latitudeRad) ) )* $earthRadius;

	}

	/**
	* west-east distance between this and the passed GeoObject
	*
	* Returns the west-east distance between this and the passed
	* object in the passed unit as float.
	*
	* @access  public
	* @param   GeoObject  &$geoObject
	* @param   int        $unit        please use _GEODB_UNIT_* constants
	* @return  float
	* @see     getDistanceNS()
	*/

	function getDistanceWE (&$geoObject, $unit = _GEODB_UNIT_DEFAULT) {

		$earthRadius = geodb::getEarthRadius ($unit);
		if ($this->longitudeRad>$geoObject->longitudeRad) {
			$direction = -1;
		} else {
			$direction = 1;
		}
		return $direction*acos (pow (sin ($this->latitudeRad), 2)+ (pow (cos ($this->latitudeRad), 2)*cos ($this->longitudeRad- $geoObject->longitudeRad) ) )* $earthRadius;

	}

	/**
	* Returns orientation between this and the passed GeoObject
	*
	* Returns the orientation of the parametric GeoObject to this,
	* like "GeoObject lies to the north of this"
	*
	* @access  public
	* @param   object  &$geoObject
	* @param   int     $form        _GEODB_ORIENTATION_SHORT or _GEODB_ORIENTATION_LONG
	* @return  string
	*/

	function getOrientation (&$geoObject, $form = _GEODB_ORIENTATION) {

		global $cfgStrings;

		$x = $this->getDistanceWE ($geoObject);
		$y = $this->getDistanceNS ($geoObject);
		$angle = rad2deg (atan2 ($y, $x) );
		if ( ($angle>67.5) && ($angle<=112.5) ) {
			return $cfgStrings[$form][0];
		}
		if ( ($angle>22.5) && ($angle<=67.5) ) {
			return $cfgStrings[$form][1];
		}
		if ( ($angle<=22.5) && ($angle>-22.5) ) {
			return $cfgStrings[$form][2];
		}
		if ( ($angle<=-22.5) && ($angle>-67.5) ) {
			return $cfgStrings[$form][3];
		}
		if ( ($angle<=-67.5) && ($angle>-112.5) ) {
			return $cfgStrings[$form][4];
		}
		if ( ($angle<=-112.5) && ($angle>-157.5) ) {
			return $cfgStrings[$form][5];
		}
		if ( ($angle>157.5) || ($angle<=-157.5) ) {
			return $cfgStrings[$form][6];
		}
		if ( ($angle>112.5) && ($angle<=157.5) ) {
			return $cfgStrings[$form][7];
		}
		return '';

	}

	/**
	* Returns an RDF-point entry
	*
	* Returns an RDF-point entry as described here: http://www.w3.org/2003/01/geo/
	* respective as shownhere, with label/name: http://www.w3.org/2003/01/geo/test/xplanet/la.rdf
	* or, with multiple entries, here: http://www.w3.org/2003/01/geo/test/towns.rdf
	* Use getRDFDataFile() for a document with header and footer.
	* The indent-paramter allows a better structure.
	*
	* @access  public
	* @param   int     $indent
	* @return  string
	* @see     getRDFDataFile()
	*/

	function getRDFPointEntry ($indent = 0) {

		$indentString = str_repeat ("\t", $indent);
		$rdfEntry = $indentString . "<geo:Point>\n";
		$rdfEntry .= $indentString . "\t<rdfs:label>" . utf8_encode ($this->name) . "</rdfs:label>\n";
		$rdfEntry .= $indentString . "\t<geo:lat>" . $this->latitude . "</geo:lat>\n";
		$rdfEntry .= $indentString . "\t<geo:long>" . $this->longitude . "</geo:long>\n";
		$rdfEntry .= $indentString . "</geo:Point>\n";
		return $rdfEntry;

	}

	/**
	* Returns an RDF-Data-File
	*
	* Returns an RDF-Data-File as described here: http://www.w3.org/2003/01/geo/
	* respective as shownhere, with label/name: http://www.w3.org/2003/01/geo/test/xplanet/la.rdf
	* or, with multiple entries, here: http://www.w3.org/2003/01/geo/test/towns.rdf
	* The only entry is this Object.
	*
	* @access  public
	* @param   int
	* @return  string
	* @see     getRDFPointEntry()
	*/

	function getRDFDataFile () {

		$rdfData = "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n";
		$rdfData .= "\txmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\"\n";
		$rdfData .= "\txmlns:geo=\"http://www.w3.org/2003/01/geo/wgs84_pos#\">\n\n";
		$rdfData .= $this->getRDFPointEntry (1);
		$rdfData .= "\n</rdf:RDF>";
		return $rdfData;

	}

	/**
	* Returns a short info about the GeoObject
	*
	* @access  public
	* @return  string
	*/

	function getInfo () {
		return $this->name . ' (' . $this->latitude . '/' . $this->longitude . ')';

	}

}

?>